<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmZfMvc
 */

namespace DeubertIt\ElasticApmZfMvc\Controller\Plugin;

use DeubertIt\ElasticApmDoctrine\Listener\ElasticApmDoctrineListener;
use DeubertIt\ElasticApmZfMvc\ElasticApmZfMvc;
use Zend_Controller_Request_Abstract;
use DeubertIt\ElasticApmDoctrine\ElasticApmDoctrine;

/**
 * Class ElasticApmZfMvcPlugin
 * @package DeubertIt\ElasticApmZfMvc\Controller\Plugin
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class ElasticApmZfMvcPlugin extends \Zend_Controller_Plugin_Abstract
{
    /**
     * @param \Zend_Controller_Request_Abstract|\Zend_Controller_Request_Http $request
     */
    public function preDispatch(\Zend_Controller_Request_Abstract $request)
    {
        $isRedirect = $this->getResponse()->isRedirect();
        $isException = $this->getResponse()->isException();

        // do not start transaction if the response is exception or is redirect
        if (ElasticApmZfMvc::isEnabled() && !$isRedirect && !$isException) {
            $name = ElasticApmZfMvc::getNameFromRequest($request);
            $transaction = ElasticApmZfMvc::start($name);

            // @todo maybe we can add Base::ELASTIC_APM_MAIN_TRANSACTION_REGISTRYKEY
            // set main transaction as parent
            if (\Zend_Registry::isRegistered('elastic-apm-agent_maintransaction')) {
                $parentTransaction = \Zend_Registry::get('elastic-apm-agent_maintransaction');
                $transaction->setParent($parentTransaction);
            }
        }

        parent::preDispatch($request);
    }

    public function postDispatch(\Zend_Controller_Request_Abstract $request)
    {
        parent::postDispatch($request);

        $isRedirect = $this->getResponse()->isRedirect();
        $isException = $this->getResponse()->isException();

        // do not try stop transaction if the response is exception or is redirect
        if (ElasticApmZfMvc::isEnabled() && !$isRedirect && !$isException) {
            $name = ElasticApmZfMvc::getNameFromRequest($request);
            $transaction = ElasticApmZfMvc::getAgent()->getTransaction($name);

            if (class_exists(ElasticApmDoctrine::class) && ElasticApmDoctrine::isEnabled()) {
                $context = ElasticApmDoctrineListener::getContext();
                $transaction->setCustomContext($context);
            }

            ElasticApmZfMvc::stop($transaction);
        }
    }
}
