<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmZfMvc
 */

namespace DeubertIt\ElasticApmZfMvc;

use DeubertIt\ElasticApm\Base;
use DeubertIt\ElasticApmZfMvc\Controller\Plugin\ElasticApmZfMvcPlugin;
use PhilKra\Agent;
use PhilKra\Events\Transaction;
use PhilKra\Exception\Transaction\UnknownTransactionException;

/**
 * Class ElasticApmZfMvc
 * @package DeubertIt\ElasticApmZfMvc
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class ElasticApmZfMvc
{
    /**
     * @var boolean
     */
    protected static $enabled = false;

    /**
     * @var Agent
     */
    protected static $agent;

    /**
     * @var Transaction|null
     */
    protected static $transaction;

    /**
     * @var Transaction|null
     */
    protected static $parentTransaction;

    protected static function registerPlugin()
    {
        $frontController = \Zend_Controller_Front::getInstance();
        $plugin = new ElasticApmZfMvcPlugin();
        if (!$frontController->hasPlugin(ElasticApmZfMvcPlugin::class)) {
            $frontController->registerPlugin($plugin);
        }
    }

    protected static function unregisterPlugin()
    {
        $frontController = \Zend_Controller_Front::getInstance();
        $plugin = new ElasticApmZfMvcPlugin();
        if ($frontController->hasPlugin(ElasticApmZfMvcPlugin::class)) {
            $frontController->unregisterPlugin($plugin);
        }
    }

    /**
     * @return boolean
     */
    public static function isEnabled()
    {
        return self::$enabled;
    }

    public static function enable()
    {
        self::$enabled = true;
        self::registerPlugin();
    }

    public static function disable()
    {
        self::$enabled = false;
        self::unregisterPlugin();
    }

    /**
     * @return Agent
     */
    public static function getAgent()
    {
        if (is_null(self::$agent)) {
            self::$agent = Base::getRegistryInstance();
        }
        return self::$agent;
    }

    /**
     * @param Agent|null $agent
     */
    public static function setAgent(Agent $agent = null)
    {
        self::$agent = $agent;
    }

    /**
     * @param string $name
     * @param Transaction|null $parent
     * @return Transaction
     */
    public static function start(string $name, Transaction $parent = null)
    {
        try {
            self::$transaction = self::getAgent()->startTransaction($name);
            if (!is_null($parent)) {
                self::$transaction->setParent($parent);
            }

            return self::$transaction;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param string|Transaction $transaction
     */
    public static function stop($transaction)
    {
        try {
            $name = $transaction;
            if ($transaction instanceof Transaction) {
                $name = $transaction->getTransactionName();
            }

            self::getAgent()->stopTransaction($name);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \Zend_Controller_Request_Abstract|\Zend_Controller_Request_Http $request
     * @return string
     */
    public static function getNameFromRequest(\Zend_Controller_Request_Abstract $request)
    {
        return $request->getMethod() . ' ' . $request->getRequestUri();
    }

    /**
     * @return Transaction|null
     */
    public static function getCurrentTransaction()
    {
        return self::$transaction;
    }

    /**
     * @return bool
     */
    public static function transactionAlreadyExists(string $name = null)
    {
        if (is_null($name)) {
            return false;
        }

        try {
            /** @var Transaction $transaction */
            $transaction = self::getAgent()->getTransaction($name);
            if ($transaction && $transaction instanceof Transaction) {
                return true;
            }
        } catch (UnknownTransactionException $e) {}

        return false;
    }
}
